#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>

#include "movie.hpp"
#include "util.hpp"
#include "types.hpp"

using namespace std;

namespace{
    const char FCM_STATE_NULL[] = {
        'F', 'C', 'S',  // "FCS"
        '\x00',         // o[Wԍ0
        '\x00', '\x00', '\x00', '\x00', // wb_TCY0
        '\x00', '\x00', '\x00', '\x00', // Vo[Wԍ0
        '\x00', '\x00', '\x00', '\x00', // \̈(0tB)
    };

    void error(const string& msg)
    {
        cerr << msg << endl;
        exit(1);
    }

    void usage()
    {
        error("Usage: fcmoptim <input FCM> <output FCM>");
    }
}

int main(int argc, char** argv)
{
    if(argc != 3) usage();

    ifstream in(argv[1], ios_base::in | ios_base::binary);
    if(!in) error("Can't open input file");
    FCMInfo info;
    string state;
    EmuInputSeq inputs;
    try{
        fcm_read(in, info, state, inputs);
    }
    catch(const MovieError& e){
        error(e.what());
    }
    in.close();

    // Xe[gx[XłȂꍇnullXe[gɒu
    if(info.from != MOVIE_FROM_STATE)
        state.assign(FCM_STATE_NULL, sizeof(FCM_STATE_NULL));

    ofstream out(argv[2], ios_base::out | ios_base::trunc | ios_base::binary);
    if(!out) error("Can't open output file");
    try{
        fcm_write(out, info, state, inputs);
    }
    catch(const MovieError& e){
        error(e.what());
    }
    out.close();

    return 0;
}
