#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <ctime>

#include "movie.hpp"
#include "util.hpp"
#include "types.hpp"

using namespace std;

namespace{
    void warn(const string& msg)
    {
        cerr << msg << endl;
    }

    void error(const string& msg)
    {
        warn(msg);
        exit(1);
    }

    /**
     * ̐@͐Ȃ̂ȂAFCEUXłl̃R[hɂȂĂ
     */
    void uuid_gen(UUID& uuid)
    {
        uchar uuid_raw[16];
        srand(time(NULL));
        for(int i = 0; i < 16; ++i)
            uuid_raw[i] = rand();
        uuid_from_raw(uuid, uuid_raw);
    }

    void usage()
    {
        error("Usage: fcmtofm2 <FCM> <FM2>");
    }
}

int main(int argc, char** argv)
{
    if(argc != 3) usage();

    FCMInfo info_fcm;
    string state;
    EmuInputSeq inputs;

    ifstream in(argv[1], ios_base::in | ios_base::binary);
    if(!in) error("Can't open input file");
    try{
        fcm_read(in, info_fcm, state, inputs);
    }
    catch(const MovieError& e){
        error(e.what());
    }
    in.close();

    if(info_fcm.from == MOVIE_FROM_STATE) error("Can't convert state-based FCM. Sorry");
    if(info_fcm.from == MOVIE_FROM_RESET) warn("[WARNING] Reset-based FCM. Converting to power-based FM2");
    if(info_fcm.synchack) warn("[WARNING] Sync-hacked FCM. Converting anyway");

    FM2Info info_fm2;
    info_fm2.from = MOVIE_FROM_POWER;
    info_fm2.pal = info_fcm.pal;
    info_fm2.rerecord = info_fcm.rerecord;
    info_fm2.emu_ver = info_fcm.emu_ver;
    info_fm2.rom_md5 = info_fcm.rom_md5;
    info_fm2.rom_name = info_fcm.rom_name;
    uuid_gen(info_fm2.uuid);

    ofstream out(argv[2], ios_base::out | ios_base::trunc | ios_base::binary);
    if(!out) error("Can't open output file");
    try{
        fm2_write(out, info_fm2, string(), inputs);
    }
    catch(const MovieError& e){
        error(e.what());
    }
    out.close();

    return 0;
}
