#include <iostream>
#include <fstream>
#include <cstdlib>

#include "movie.hpp"
#include "util.hpp"
#include "types.hpp"

using namespace std;

namespace{
    const char FCM_STATE_NULL[] = {
        'F', 'C', 'S',  // "FCS"
        '\x00',         // o[Wԍ0
        '\x00', '\x00', '\x00', '\x00', // wb_TCY0
        '\x00', '\x00', '\x00', '\x00', // Vo[Wԍ0
        '\x00', '\x00', '\x00', '\x00', // \̈(0tB)
    };

    const ulong FCEU_VER_DEF = 9828;

    void warn(const string& msg)
    {
        cerr << msg << endl;
    }

    void error(const string& msg)
    {
        warn(msg);
        exit(1);
    }

    void usage()
    {
        error("Usage: fm2tofcm <FM2> <FCM>");
    }
}

int main(int argc, char** argv)
{
    if(argc != 3) usage();

    FM2Info info_fm2;
    string state;
    EmuInputSeq inputs;

    ifstream in(argv[1], ios_base::in | ios_base::binary);
    if(!in) error("Can't open input file");
    try{
        fm2_read(in, info_fm2, state, inputs);
    }
    catch(const MovieError& e){
        error(e.what());
    }
    in.close();

    if(info_fm2.from == MOVIE_FROM_STATE) error("Can't convert state-based FCM. Sorry");

    FCMInfo info_fcm;
    info_fcm.from = MOVIE_FROM_POWER;
    info_fcm.pal = info_fm2.pal;
    info_fcm.synchack = false;
    info_fcm.rerecord = info_fm2.rerecord;
    info_fcm.emu_ver = FCEU_VER_DEF; // FCEUX̃o[Ŵ܂ܓ̂͂܂Ȃ̂Łc
    info_fcm.rom_md5 = info_fm2.rom_md5;
    info_fcm.rom_name = info_fm2.rom_name;
    info_fcm.metadata = "";
    state.assign(FCM_STATE_NULL, sizeof(FCM_STATE_NULL));

    ofstream out(argv[2], ios_base::out | ios_base::trunc | ios_base::binary);
    if(!out) error("Can't open output file");
    try{
        fcm_write(out, info_fcm, state, inputs);
    }
    catch(const MovieError& e){
        error(e.what());
    }
    out.close();

    return 0;
}
