#ifndef FCEUTOOLS_MOVIE_HPP
#define FCEUTOOLS_MOVIE_HPP

#include <iostream>
#include <string>
#include <vector>
#include <stdexcept>

#include "util.hpp"
#include "types.hpp"

enum EmuCmd{
    EMU_CMD_UNKNOWN,
    EMU_CMD_NONE,
    EMU_CMD_RESET,
    EMU_CMD_POWER,
    EMU_CMD_FDSINSERT,
    EMU_CMD_FDSSELECT,
};

const uint JOY_A = 0x01;
const uint JOY_B = 0x02;
const uint JOY_S = 0x04;
const uint JOY_T = 0x08;
const uint JOY_U = 0x10;
const uint JOY_D = 0x20;
const uint JOY_L = 0x40;
const uint JOY_R = 0x80;

struct EmuInput{
    EmuCmd cmd;
    uchar joy[4];
};

typedef std::vector<EmuInput> EmuInputSeq;

enum MovieFrom{
    MOVIE_FROM_RESET,
    MOVIE_FROM_POWER,
    MOVIE_FROM_STATE,
};

struct FCMInfo{
    MovieFrom from;
    bool pal;
    bool synchack;
    ulong rerecord;
    ulong emu_ver;
    MD5 rom_md5;
    std::string rom_name;
    std::string metadata;
};

struct FM2Info{
    MovieFrom from;
    bool pal;
    ulong rerecord;
    ulong emu_ver;
    MD5 rom_md5;
    std::string rom_name;
    UUID uuid;
};

class MovieError : public std::runtime_error{
public:
    explicit MovieError(const std::string& msg);
};

void fcm_read(std::istream& in, FCMInfo& info, std::string& state, EmuInputSeq& inputs);
void fcm_write(std::ostream& out, const FCMInfo& info, const std::string& state, const EmuInputSeq& inputs);

void fm2_read(std::istream& in, FM2Info& info, std::string& state, EmuInputSeq& inputs);
void fm2_write(std::ostream& out, const FM2Info& info, const std::string& state, const EmuInputSeq& inputs);

#endif /* FCEUTOOLS_MOVIE_HPP */
